﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows.Media;
using System.Windows.Threading;

namespace CassiniSwitch.ViewModel
{
	abstract class ViewModelBase : INotifyPropertyChanged
	{
		#region Common view properties

		private string displayName;
		/// <summary>
		/// Gets or sets the user-friendly name of this object. Inherited classes can set this
		/// property to a new value, or override it to determine the value on-demand.
		/// </summary>
		public virtual string DisplayName
		{
			get { return displayName; }
			set
			{
				if (value != displayName)
				{
					displayName = value;
					OnPropertyChanged("DisplayName");
				}
			}
		}

		#endregion Common view properties

		#region Data input cleanup

		protected string SanitizeString(string str)
		{
			if (str == null) return null;
			return str.Trim();
		}

		protected string SanitizeInt(string str)
		{
			if (string.IsNullOrWhiteSpace(str)) return null;
			try
			{
				long l = Convert.ToInt64(str);
				return Convert.ToString(l);
			}
			catch
			{
				return str.Trim();
			}
		}

		protected string SanitizeDouble(string str)
		{
			//return str.Trim();
			if (string.IsNullOrWhiteSpace(str)) return null;
			try
			{
				double d = Convert.ToDouble(str);
				return Convert.ToString(d);
			}
			catch
			{
				return str.Trim();
			}
		}

		#endregion Data input cleanup

		#region INotifyPropertyChanged Member

		/// <summary>
		/// Raised when a property on this object has a new value.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		/// Raises this object's PropertyChanged event.
		/// </summary>
		/// <param name="propertyName">The property that has a new value.</param>
		protected void OnPropertyChanged(string propertyName)
		{
#if DEBUG
			if (!TypeDescriptor.GetProperties(this).OfType<PropertyDescriptor>().Any(d => d.Name == propertyName))
			{
				throw new NotImplementedException("Notifying a change of non-existing property " + this.GetType().Name + "." + propertyName);
			}
#endif

			var handler = PropertyChanged;
			if (handler != null)
			{
				handler(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion INotifyPropertyChanged Member
	}
}
